@@ -1,20 +1,22 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2018 by Dominique Dumont.
+# This software is Copyright (c) 2005-2021 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::Node;
-$Config::Model::Node::VERSION = '2.133';
+package Config::Model::Node 2.141;
+
 use Mouse;
 with "Config::Model::Role::NodeLoader";
 
 use Carp;
 use 5.010;
 
+use Config::Model::TypeConstraints;
+use Config::Model::Instance;
 use Config::Model::Exception;
 use Config::Model::Loader;
 use Config::Model::Dumper;
@@ -95,7 +97,8 @@ sub fetch_gist {
     return $gist;
 }
 
-has [qw/config_file element_name/] => ( is => 'ro', isa => 'Maybe[Str]', required => 0 );
+has config_file => ( is => 'ro', isa => 'Config::Model::TypeContraints::Path', required => 0 );
+has element_name => ( is => 'ro', isa => 'Maybe[Str]', required => 0 );
 
 has instance => (
     is => 'ro',
@@ -123,7 +126,19 @@ has needs_save => ( is => 'rw', isa => 'Bool', default => 0 );
 
 has backend_mgr => ( is => 'ro', isa => 'Maybe[Config::Model::BackendMgr]' );
 
-# attribute is defined in Config::Model::Anythin
+# used to avoid warning twice about a deprecated element. Internal methods
+has warned_deprecated_element => (
+    is => 'ro',
+    isa => 'HashRef[Str]',
+    traits     => ['Hash'],
+    default => sub { {}; },
+    handles => {
+        warn_element_done => 'set',
+        was_element_warned => 'defined',
+    }
+) ;
+
+# attribute is defined in Config::Model::Anything
 sub _backend_support_annotation {
     my $self = shift;
     return $self->backend_mgr ? $self->backend_mgr->support_annotation
@@ -139,10 +154,8 @@ sub BUILD {
     my $class_name = $self->config_class_name;
     $logger->debug("New $class_name requested by $caller_class");
 
-    # get_model returns a cloned data structure
-    $self->model( $self->config_model->get_model($class_name) );
-
-    $self->{original_model} = dclone($self->model);
+    $self->{original_model} = $self->config_model->model($class_name);
+    $self->model( dclone($self->{original_model}) ) ;
 
     $self->check_properties;
 
@@ -425,6 +438,7 @@ sub has_element {
     my %args = ( @_ > 1 ) ? @_ : ( name => shift );
     my $name = $args{name};
     my $type = $args{type};
+    my $autoadd = $args{autoadd} // 1;
 
     if ( not defined $name ) {
         Config::Model::Exception::Internal->throw(
@@ -433,7 +447,7 @@ sub has_element {
         );
     }
 
-    $self->accept_element($name);
+    $self->accept_element($name) if $autoadd;
     return 0 unless defined $self->{model}{element}{$name};
     return 1 unless defined $type;
     return $self->{model}{element}{$name}{type} eq $type ? 1 : 0;
@@ -641,6 +655,7 @@ sub fetch_element {
 
     my $check         = $self->_check_check( $args{check} );
     my $accept_hidden = $args{accept_hidden} || 0;
+    my $autoadd       = $args{autoadd} // 1;
 
     $self->init();
 
@@ -650,7 +665,7 @@ sub fetch_element {
     if ( not defined $self->{element}{$element_name} ) {
 
         # We also need to check if element name is matched by any of 'accept' parameters
-        $self->accept_element($element_name);
+        $self->accept_element($element_name) if $autoadd;
         $self->create_element( name => $element_name, check => $check ) or return;
     }
 
@@ -685,16 +700,12 @@ sub fetch_element {
     # do not warn when when is skip or "no"
     if ($self->{status}{$element_name} eq 'deprecated' and $check eq 'yes' ) {
         # FIXME elaborate more ? or include parameter description ??
-        if ($::_use_log4perl_to_warn) {
-            $user_logger->warn(
-                "Element '$element_name' of node '", $self->name, "' is deprecated"
-            )
-                if $check eq 'yes';
+        my $msg = "Element '$element_name' of node '". $self->name. "' is deprecated";
+        if (not $self->was_element_warned($element_name)) {
+            if ($::_use_log4perl_to_warn) { $user_logger->warn($msg); }
+            else                          { warn("$msg\n"); }
+            $self->warn_element_done($element_name,1);
         }
-        else {
-            warn("Element '$element_name' of node '", $self->name, "' is deprecated\n") if $check eq 'yes';
-        }
-
         # this will also force a rewrite of the file even if no other
         # semantic change was done
         $self->notify_change(
@@ -969,6 +980,7 @@ sub load_data {
         ") will load elt ",
         join( ' ', sort keys %$perl_data ) );
 
+    my $has_stored = 0;
     # data must be loaded according to the element order defined by
     # the model. This will not load not yet accepted parameters
     foreach my $elt ( @{ $self->{model}{element_list} } ) {
@@ -987,7 +999,7 @@ sub load_data {
             );
 
             if ($obj) {
-                $obj->load_data( %args, data => delete $perl_data->{$elt} );
+                $has_stored += $obj->load_data( %args, data => delete $perl_data->{$elt} );
             }
             elsif ( defined $obj ) {
 
@@ -1021,7 +1033,7 @@ sub load_data {
             my $obj = $self->fetch_element( name => $elt, check => $check );
             next unless $obj;    # in cas of known but unavailable elements
             $logger->info("Node load_data: accepting element $elt");
-            $obj->load_data( %args, data => delete $perl_data->{$elt} ) if defined $obj;
+            $has_stored += $obj->load_data( %args, data => delete $perl_data->{$elt} ) if defined $obj;
         }
     }
 
@@ -1033,16 +1045,20 @@ sub load_data {
             object     => $self,
         );
     }
+    return !! $has_stored;
 }
 
-# TBD explain full_dump
-
 sub dump_tree {
     my $self = shift;
     my %args = @_;
     $self->init();
+    my $full = delete $args{full_dump} || 0;
+    if ($full) {
+        carp "dump_tree: full_dump parameter is deprecated. Please use 'mode => user' instead";
+        $args{mode} //= 'user';
+    }
     my $dumper = Config::Model::Dumper->new;
-    $dumper->dump_tree( node => $self, @_ );
+    $dumper->dump_tree( node => $self, %args );
 }
 
 sub migrate {
@@ -1122,6 +1138,19 @@ sub get_help {
     return defined $help ? $help : '';
 }
 
+sub get_info {
+    my $self = shift;
+
+    my @items = ( 'type: ' . $self->get_type, 'class name: ' . $self->config_class_name, );
+
+    my @rexp = $self->accept_regexp;
+    if (@rexp) {
+        push @items, 'accept: /^' . join( '$/, /^', @rexp ) . '$/';
+    }
+
+    return @items;
+}
+
 sub tree_searcher {
     my $self = shift;
 
@@ -1222,7 +1251,7 @@ Config::Model::Node - Class for configuration tree node
 
 =head1 VERSION
 
-version 2.133
+version 2.141
 
 =head1 SYNOPSIS
 
@@ -1464,7 +1493,7 @@ B<type> parameter. The I<type> type can be:
 
 =item C<node>
 
-The element is a simple node of a tree instantiated from a
+The element is a node of a tree instantiated from a
 configuration class (declared with
 L<Config::Model/"create_config_class( ... )">).
 See L</"Node element">.
@@ -1592,11 +1621,14 @@ L<Config::Model::AnyThing>
 
 =head2 has_element
 
-Parameters: C<< ( name => element_name, [ type => searched_type ] ) >>
+Arguments: C<< ( name => element_name, [ type => searched_type ],  [ autoadd => 1 ] ) >>
 
-Returns 1 if the class model has the element declared or if the element
-name is matched by the optional C<accept> parameter. If C<type> is specified, the
-element name must also match the type.
+Returns 1 if the class model has the element declared.
+
+Returns 1 as well if C<autoadd> is 1 (i.e. by default) and the element
+name is matched by the optional C<accept> model parameter.
+
+If C<type> is specified, the element name must also match the type.
 
 =head2 find_element
 
@@ -1613,7 +1645,7 @@ Returns empty if no matching element is found.
 =head2 model_searcher
 
 Returns an object dedicated to search an element in the configuration
-model (respecting privilege level).
+model.
 
 This method returns a L<Config::Model::SearchElement> object. See
 L<Config::Model::SearchElement> for details on how to handle a search.
@@ -1757,13 +1789,22 @@ Reset a property of an element according to the original model.
 
 =head2 fetch_element
 
-Parameters: C<< ( name => .. , [ check => ..] ) >>
+Arguments: C<< ( name => .. , [ check => ..], [ autoadd => 1 ] ) >>
 
-Fetch and returns an element from a node.
+Fetch and returns an element from a node if the class model has the
+element declared.
 
-check can be set to yes, no or skip. When check is C<no> or C<skip>, can return C<undef> when the
+Also fetch and returns an element from a node if C<autoadd> is 1
+(i.e. by default) and the element name is matched by the optional
+C<accept> model parameter.
+
+C<check> can be set to C<yes>, C<no> or C<skip>.
+When C<check> is C<no> or C<skip>, this method returns C<undef> when the
 element is unknown, or 0 if the element is not available (hidden).
 
+By default, "accepted" elements are automatically created. Set
+C<autoadd> to 0 when this behavior is not wanted.
+
 =head2 fetch_element_value
 
 Parameters: C<< ( name => ... [ check => ...] ) >>
@@ -1891,6 +1932,8 @@ Use C<< check => skip >> to make data loading more tolerant: bad data are discar
 
 C<load_data> can be called with a single hash ref parameter.
 
+Returns 1 if some data were saved (instead of skipped).
+
 =head2 needs_save
 
 return 1 if one of the elements of the node's sub-tree has been modified.
@@ -1954,6 +1997,11 @@ C<description> of the element.
 
 Returns an empty string if no description was found.
 
+=head2 get_info
+
+Returns a list of information related to the node. See
+L<Config::Model::Value/get_info> for more details.
+
 =head2 tree_searcher
 
 Parameters: C<< ( type => ... ) >>
@@ -2001,7 +2049,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2018 by Dominique Dumont.
+This software is Copyright (c) 2005-2021 by Dominique Dumont.
 
 This is free software, licensed under:
 
