@@ -1,13 +1,13 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2019 by Dominique Dumont.
+# This software is Copyright (c) 2005-2021 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::Value 2.136;
+package Config::Model::Value 2.141;
 
 use 5.10.1;
 
@@ -126,6 +126,7 @@ has warning_list => (
         clear_warnings => 'clear',
         warning_msg    => [ join => "\n\t" ],
         has_warning    => 'count',
+        has_warnings    => 'count',
         all_warnings   => 'elements',
     } );
 
@@ -299,11 +300,11 @@ sub perform_compute {
     my $result = $self->compute_obj->compute;
 
     # check if the computed result fits with the constraints of the
-    # Value object
-    my $ok = $self->check_fetched_value($result);
+    # Value model
+    my ($ok, $value, $error, $warn) = $self->_check_value(value => $result);
 
     if ( not $ok ) {
-        my $error = $self->error_msg . "\n\t" . $self->compute_info;
+        my $error = join("\n", (@$error, $self->compute_info));
 
         Config::Model::Exception::WrongValue->throw(
             object => $self,
@@ -405,15 +406,16 @@ sub setup_enum_choice {
     # whether a value is present in the enum set is easier
     delete $self->{choice_hash} if defined $self->{choice_hash};
 
-    map { $self->{choice_hash}{$_} = 1; } @choice;
+    for ( @choice ) { $self->{choice_hash}{$_} = 1; }
 
     # delete the current value if it does not fit in the new
     # choice
-    map {
+    for ( qw/data preset/ ) {
         my $lv = $self->{$_};
-
-        delete $self->{$_} if ( defined $lv and not defined $self->{choice_hash}{$lv} );
-    } qw/data preset/;
+        if ( defined $lv and not defined $self->{choice_hash}{$lv} ) {
+            delete $self->{$_};
+        }
+    }
 }
 
 sub setup_match_regexp {
@@ -524,13 +526,13 @@ sub set_properties {
     my $self = shift;
 
     # cleanup all parameters that are handled by warp
-    map( delete $self->{$_}, @allowed_warp_params );
+    for ( @allowed_warp_params ) { delete $self->{$_} }
 
     # merge data passed to the constructor with data passed to set_properties
     my %args = ( %{ $self->{backup} }, @_ );
 
     # these are handled by Node or Warper
-    map { delete $args{$_} } qw/level/;
+    for ( qw/level/ ) { delete $args{$_} }
 
     if ( $logger->is_trace ) {
         $logger->trace( "Leaf '" . $self->name . "' set_properties called with '",
@@ -548,9 +550,10 @@ sub set_properties {
         );
     }
 
-    map { $self->{$_} = delete $args{$_} if defined $args{$_} }
-        qw/min max mandatory warn replace_follow assert warn_if warn_unless
-        write_as/;
+    for (qw/min max mandatory warn replace_follow assert warn_if warn_unless
+            write_as/) {
+        $self->{$_} = delete $args{$_} if defined $args{$_};
+    }
 
     if ($args{replace}) {
         $self->{replace} = delete $args{replace};
@@ -702,7 +705,13 @@ sub get_cargo_type {
 sub can_store {
     my $self = shift;
 
-    return not defined $self->compute || $self->compute_obj->allow_user_override;
+    if ( not defined $self->compute ) {
+        return 1;
+    }
+    if ( $self->compute_obj->allow_user_override ) {
+        return 1;
+    }
+    return;
 }
 
 sub get_default_choice {
@@ -715,12 +724,54 @@ sub get_choice {
 
     # just in case the reference_object has been changed
     if ( defined $self->{refer_to} or defined $self->{computed_refer_to} ) {
-        $self->{ref_object}->get_choice_from_refered_to;
+        $self->{ref_object}->get_choice_from_referred_to;
     }
 
     return @{ $self->{choice} || [] };
 }
 
+sub get_info {
+    my $self = shift;
+
+    my $type       = $self->value_type;
+    my @choice     = $type eq 'enum' ? $self->get_choice : ();
+    my $choice_str = @choice ? ' (' . join( ',', @choice ) . ')' : '';
+
+    my @items = ( 'type: ' . $self->value_type . $choice_str, );
+
+    my $std = $self->fetch(qw/mode standard check no/);
+
+    if ( defined $self->upstream_default ) {
+        push @items, "upstream_default value: " . $self->upstream_default;
+    }
+    elsif ( defined $std ) {
+        push @items, "default value: $std";
+    }
+    elsif ( defined $self->refer_to ) {
+        push @items, "reference to: " . $self->refer_to;
+    }
+    elsif ( defined $self->computed_refer_to ) {
+        push @items, "computed reference to: " . $self->computed_refer_to;
+    }
+
+    my $m = $self->mandatory;
+    push @items, "is mandatory: " . ( $m ? 'yes' : 'no' ) if defined $m;
+
+    foreach my $what (qw/min max warn grammar/) {
+        my $v = $self->$what();
+        push @items, "$what value: $v" if defined $v;
+    }
+
+    foreach my $what (qw/warn_if_match warn_unless_match/) {
+        my $v = $self->$what();
+        foreach my $k ( keys %$v ) {
+            push @items, "$what value: $k";
+        }
+    }
+
+    return @items ;
+}
+
 sub get_help {
     my $self = shift;
 
@@ -749,20 +800,20 @@ sub enum_error {
         return @error;
     }
 
-    my @choice    = map( "'$_'", $self->get_choice );
+    my @choice    = map { "'$_'" } $self->get_choice;
     my $var       = $self->{value_type};
     my $str_value = defined $value ? $value : '<undef>';
     push @error,
         "$self->{value_type} type does not know '$value'. Expected " . join( " or ", @choice );
     push @error,
-        "Expected list is given by '" . join( "', '", @{ $self->{refered_to_path} } ) . "'"
-        if $var eq 'reference' && defined $self->{refered_to_path};
+        "Expected list is given by '" . join( "', '", @{ $self->{referred_to_path} } ) . "'"
+        if $var eq 'reference' && defined $self->{referred_to_path};
     push @error, $self->warp_error if $self->{warp};
 
     return @error;
 }
 
-sub check_value {
+sub _check_value {
     my $self = shift;
     croak "check_value needs a value to check" unless @_ > 1;
 
@@ -772,9 +823,6 @@ sub check_value {
     my $check     = $args{check} || 'yes';
     my $apply_fix = $args{fix} || 0;
     my $mode      = $args{mode} || 'backend';
-    my $cb        = delete $args{callback};
-
-    carp "callback parameter is deprecated" if defined $cb;
 
     #croak "Cannot specify a value with fix = 1" if $apply_fix and exists $args{value} ;
 
@@ -831,7 +879,7 @@ sub check_value {
 
         # just in case the reference_object has been changed
         if ( defined $self->{refer_to} or defined $self->{computed_refer_to} ) {
-            $self->{ref_object}->get_choice_from_refered_to;
+            $self->{ref_object}->get_choice_from_referred_to;
         }
 
         if (    length($value)
@@ -944,17 +992,24 @@ sub check_value {
         scalar @error,
         " errors and ", scalar @warn, " warnings"
     );
+
+    my $ok = not @error;
+    return ($ok, $value, \@error, \@warn);
+}
+
+
+sub check_value {
+    my $self = shift;
+
+    my ($ok, $value, $error, $warn) = $self->_check_value(@_);
     $self->clear_errors;
     $self->clear_warnings;
-    $self->add_error(@error)  if @error;
-    $self->add_warning(@warn) if @warn;
-
-    $args{value} = $value;    # may be updated by apply_fix
+    $self->add_error(@$error)  if @$error;
+    $self->add_warning(@$warn) if @$warn;
 
     $logger->trace("done");
 
-    $cb->( %args, ok => not @error ) if $cb;
-    my $ok = not @error;
+    # return $value because it may be updated by apply_fix
     return wantarray ? ($ok, $value) : $ok;
 }
 
@@ -1023,7 +1078,7 @@ sub run_code_set_on_value {
                     message => "Eval of assert or warning code failed : $@"
                 );
             }
-            return $invert ^ $ret;
+            return ($invert xor $ret) ;
         };
 
         $self->run_code_on_value( $value_r, $apply_fix, $array, $label, $sub, $msg, $fix );
@@ -1073,6 +1128,8 @@ sub apply_fixes {
             );
         }
     } while ( $self->{nb_of_fixes} and $old > $new );
+
+    $self->show_warnings($self->_fetch_no_check);
 }
 
 # internal: called by check when a fix is required
@@ -1178,12 +1235,12 @@ sub check_fetched_value {
         $logger->debug("is not needed");
     }
 
-    $self->store_warning($value, $silent);
+    $self->show_warnings($value, $silent);
 
     return wantarray ? $self->all_errors : $self->is_ok;
 }
 
-sub store_warning {
+sub show_warnings {
     my ($self, $value, $silent) = @_ ;
 
     # old_warn is used to avoid warning the user several times for the
@@ -1193,17 +1250,23 @@ sub store_warning {
     my %warn_h;
 
     if ( $self->has_warning and not $nowarning and not $silent ) {
+        my $str = $value // '<undef>';
+        chomp $str;
+        my $w_str = $str =~ /\n/ ? "\n+++++\n$str\n+++++" : "'$str'";
         foreach my $w ( $self->all_warnings ) {
             $warn_h{$w} = 1;
-            next if $old_warn->{$w};
-            my $str = $value // '<undef>';
-            chomp $str;
-            my $w_str = $str =~ /\n/ ? "\n+++++\n$str\n+++++" : "'$str'";
-            if ($::_use_log4perl_to_warn) {
-                $user_logger->warn("Warning in '" . $self->location_short . "': $w\nOffending value: $w_str");
+            my $w_msg = "Warning in '" . $self->location_short . "': $w\nOffending value: $w_str";
+            if ($old_warn->{$w}) {
+                # user has already seen the warning, let's use debug level (required by tests)
+                $user_logger->debug($w_msg);
             }
             else {
-                warn "Warning in '" . $self->location_short . "': $w\nOffending value: $w_str\n";
+                if ($::_use_log4perl_to_warn) {
+                    $user_logger->warn($w_msg);
+                }
+                else {
+                    warn "$w_msg\n";
+                }
             }
         }
     }
@@ -1314,7 +1377,7 @@ sub _store {
     if ( $logger->is_debug ) {
         my $i   = $self->instance;
         my $msg = "value store ". ($value // '<undef>')." ok '$ok', check is $check";
-        map { $msg .= " $_" if $i->$_() } qw/layered preset/;
+        for ( qw/layered preset/ ) { $msg .= " $_" if $i->$_() }
         $logger->debug($msg);
     }
 
@@ -1372,10 +1435,10 @@ sub transform_boolean {
 
     if ( my $wa = $self->{write_as} ) {
         my $i = 0;
-        map {
+        for ( @$wa ) {
             $$v_ref = $i if ( $wa->[$i] eq $$v_ref );
             $i++
-        } @$wa;
+        }
     }
 
     # convert yes no to 1 or 0
@@ -1408,7 +1471,7 @@ sub transform_value {
     }
 
     if ( defined $self->{refer_to} or defined $self->{computed_refer_to} ) {
-        $self->{ref_object}->get_choice_from_refered_to;
+        $self->{ref_object}->get_choice_from_referred_to;
     }
 
     $value = $self->{convert_sub}($value)
@@ -1464,7 +1527,7 @@ sub check_stored_value {
     # must always warn when storing a value, hence clearing the list
     # of already issued warnings
     $self->{old_warning_hash} = {};
-    $self->store_warning($value, $silent);
+    $self->show_warnings($value, $silent);
 
     return wantarray ? ($ok,$fixed_value) : $ok;
 }
@@ -1533,7 +1596,7 @@ sub _init {
 
     if ( defined $self->{refer_to} or defined $self->{computed_refer_to} ) {
         $self->submit_to_refer_to;
-        $self->{ref_object}->get_choice_from_refered_to;
+        $self->{ref_object}->get_choice_from_referred_to;
     }
 }
 
@@ -1600,8 +1663,18 @@ my %old_mode = (
     non_built_in => 'non_upstream_default',
 );
 
-my %accept_mode = map { ( $_ => 1 ) } qw/custom standard preset default upstream_default
-    layered non_upstream_default allow_undef user backend/;
+{
+    my %accept_mode = map { ( $_ => 1 ) } qw/custom standard preset default upstream_default
+                                             layered non_upstream_default allow_undef user backend/;
+
+    sub is_bad_mode {
+        my ($self, $mode) = @_;
+        if ( $mode and not defined $accept_mode{$mode} ) {
+            my $good_ones = join( ' or ', sort keys %accept_mode );
+            return "expected $good_ones as mode parameter, not $mode";
+        }
+    }
+}
 
 sub _fetch {
     my ( $self, $mode, $check ) = @_;
@@ -1636,10 +1709,8 @@ sub _fetch {
         carp $self->location, " warning: deprecated mode parameter: $k, ", "expected $mode\n";
     }
 
-    if ( $mode and not defined $accept_mode{$mode} ) {
-        croak "fetch_no_check: expected ",
-            join( ' or ', sort keys %accept_mode ),
-            " parameter, not $mode";
+    if (my $err = $self->is_bad_mode($mode)) {
+        croak "fetch_no_check: $err";
     }
 
     if ( $mode eq 'custom' ) {
@@ -1709,6 +1780,14 @@ sub _fetch_no_check {
         :                                  $self->{default};
 }
 
+sub fetch_summary {
+    my $self = shift;
+    my $value = $self->fetch(@_) // '<undef>';
+    $value =~ s/\n/ /g;
+    $value = substr( $value, 0, 15 ) . '...' if length($value) > 15;
+    return $value;
+}
+
 sub fetch {
     my $self = shift;
 
@@ -1733,9 +1812,8 @@ sub fetch {
         $logger->debug( "_fetch returns " . ( defined $value ? $value : '<undef>' ) );
     }
 
-    if ( $mode and not defined $accept_mode{$mode} ) {
-        croak "fetch: expected ", not scalar join( ' or ', sort keys %accept_mode ),
-            " parameter, not $mode";
+    if ( my $err = $self->is_bad_mode($mode) ) {
+        croak "fetch: $err";
     }
 
     if ( defined $self->{replace_follow} and defined $value ) {
@@ -1757,7 +1835,7 @@ sub fetch {
     # mode may not trigger the warnings. Hence confusion afterwards)
     my $ok = 1;
     $ok = $self->check( value => $value, silent => $silent, mode => $mode )
-        if $mode =~ /backend|custom|user/;
+        if $mode =~ /backend|custom|user/ and $check ne 'no';
 
     $logger->trace( "$mode fetch (almost) done for " . $self->location )
         if $logger->is_trace;
@@ -1919,7 +1997,7 @@ Config::Model::Value - Strongly typed configuration value
 
 =head1 VERSION
 
-version 2.136
+version 2.141
 
 =head1 SYNOPSIS
 
@@ -2542,6 +2620,16 @@ value or undef.
 
 Without parameter returns a hash ref that contains all the help strings.
 
+=head2 get_info
+
+Returns a list of information related to the value, like value type,
+default value. This should be used to provide some debug information
+to the user.
+
+For instance, C<$val->get-info> may return:
+
+ [ 'type: string', 'mandatory: yes' ]
+
 =head2 error_msg
 
 Returns the error messages of this object (if any)
@@ -2597,6 +2685,19 @@ L</warning_msg>.
 
 Without C<value> argument, this method checks the value currently stored.
 
+=head2 is_bad_mode
+
+Accept a mode parameter. This function checks if the mode is accepted
+by L</fetch> method. Returns an error message if not. For instance:
+
+ if (my $err = $val->is_bad_mode('foo')) {
+    croak "my_function: $err";
+ }
+
+This method is intented as a helper to avoid duplicating the list of
+accepted modes for functions that want to wrap fetch methods (like
+L<Config::Model::Dumper> or L<Config::Model::DumpAsData>)
+
 =head1 Information management
 
 =head2 store
@@ -2605,8 +2706,8 @@ Parameters: C<< ( $value ) >>
 or C<< value => ...,	check => yes|no|skip ), silent => 0|1 >>
 
 Store value in leaf element. C<check> parameter can be used to
-skip validation check (default ies 'yes').
-C<silent> cane be used to suppress warnings.
+skip validation check (default is 'yes').
+C<silent> can be used to suppress warnings.
 
 Optional C<callback> is now deprecated.
 
@@ -2619,10 +2720,10 @@ computed or migrated value).
 
 Parameters: C<< ( $value ) >>
 
-Load scalar data. Data is forwarded to L</"store">.
+Called with the same parameters are C<store> method.
 
-Called with C<load_data> or C<load_data> or
-with the same parameters are C<store> method.
+Load scalar data. Data is forwarded to L</"store"> after checking that
+the passed value is not a reference.
 
 =head2 fetch_custom
 
@@ -2733,11 +2834,17 @@ feature is useful to reduce data to write in configuration file.
 
 =item allow_undef
 
-With this mode, C<fetch()> returns undef for mandatory values. Normally,
-trying to fetch an undefined mandatory value leads to an exception.
+With this mode, C<fetch()> behaves like in C<user> mode, but returns
+C<undef> for mandatory values. Normally, trying to fetch an undefined
+mandatory value leads to an exception.
 
 =back
 
+=head2 fetch_summary
+
+Returns a truncated value when the value is a string or uniline that
+is too long to be displayed.
+
 =head2 user_value
 
 Returns the value entered by the user. Does not use the default or
@@ -2989,7 +3096,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2019 by Dominique Dumont.
+This software is Copyright (c) 2005-2021 by Dominique Dumont.
 
 This is free software, licensed under:
 
