@@ -1,18 +1,17 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2018 by Dominique Dumont.
+# This software is Copyright (c) 2005-2021 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 use Config::Model::BackendMgr;
+use strict;
+use warnings;
 
-$conf_dir = '/etc';
-$conf_file_name = 'hosts.json';
-
-$model->create_config_class(
+my @config_classes = ({
     name => 'Host',
 
     element => [
@@ -22,8 +21,9 @@ $model->create_config_class(
         },
         dummy => {qw/type leaf value_type uniline default toto/},
     ]
-);
-$model->create_config_class(
+});
+
+push @config_classes, {
     name => 'Hosts',
 
     rw_config => {
@@ -41,11 +41,10 @@ $model->create_config_class(
             },
         },
     ]
-);
+};
 
-$model_to_test = "Hosts";
 
-@tests = (
+my @tests = (
     {
         name  => 'basic',
         check => [
@@ -55,4 +54,10 @@ $model_to_test = "Hosts";
     },
 );
 
-1;
+return {
+    model_to_test => "Hosts",
+    conf_dir => '/etc',
+    conf_file_name => 'hosts.json',
+    config_classes => \@config_classes,
+    tests => \@tests
+};
